from .. import schema_to_class as s2c
import json
import emod_api.campaign as camp
"""
    if(camp_obj['Nodeset_Config']['Node_List'][0] == 3):
      xval = np.arange(0,365)
      yval = 1.0+0.2*np.sin(8.0*2*np.pi*xval/365)
      camp_obj['Start_Day'] = 180
      camp_obj[ECC][IC][MBD]['Times']  = xval.tolist()
      camp_obj[ECC][IC][MBD]['Values'] = yval.tolist()

    if(camp_obj['Nodeset_Config']['Node_List'][0] == 2):
      xval = np.array([ 0.0, 50.0, 100.0, 149.5, 150.0, 200.0])
      yval = np.array([ 1.0,  1.3,   1.0,   1.0,   0.7,   0.7])
      camp_obj['Start_Day'] = 10
      camp_obj[ECC][IC][MBD]['Times']  = xval.tolist()
      camp_obj[ECC][IC][MBD]['Values'] = yval.tolist()

    if(camp_obj['Nodeset_Config']['Node_List'][0] == 1):
      xval = np.arange(0,365)
      yval = 1.0-0.8*np.exp(-5.0*xval/365)
      camp_obj['Start_Day'] = 0
      camp_obj[ECC][IC][MBD]['Times']  = xval.tolist()
      camp_obj[ECC][IC][MBD]['Values'] = yval.tolist()
"""
def new_intervention( camp, new_infectivity=1.0, profile="CONSTANT", **kwargs ):
    """
    Create new NodeInfectivityModifying intervention.
    Args:
        profile: multiplier options include:
        - CONSTANT
            - new_infectivity lasts forever (or until replaced). 
        - TRAPEZOID
            - rise duration
            - peak duration
            - fall duration
            - recurs (bool)
        - EXPONENTIAL
            - rise duration
            - rise rate
        - SINUSOIDAL
            - period

        To do boxcar, specificy 0 rise and fall durations. To do sawtooth, specify 0 peak duration.

    Returns: 
        new NodeInfectivityMult intervention dictionary.
    """
    intervention = s2c.get_class_with_defaults( "NodeInfectivityMult", camp.schema_path )
    if profile=="CONST":
        efficacy_profile = "WaningEffectBox"
        intervention.Multiplier_By_Duration.Times=[0, 36500] # I'm preferring 100 years to FLT_MAX
        intervention.Multiplier_By_Duration.Values=[new_infectivity,new_infectivity]
    elif profile=="TRAP":
        rise_dur = 0
        if "rise_dur" in kwargs:
            rise_dur = float(kwargs["rise_dur"])
        peak_dur = 0
        if "peak_dur" in kwargs:
            peak_dur = float(kwargs["peak_dur"])
        fall_dur = 0
        if "fall_dur" in kwargs:
            fall_dur = float(kwargs["fall_dur"])
        recur = true
        if "recur" in kwargs: # only needed if false, but read value anyway
            recur = bool(kwargs["recur"])
        intervention.Multiplier_By_Duration.Times=[0, rise_dur, rise_dur+peak_dur, rise_dur+peak_dur+fall_dur]
        intervention.Multiplier_By_Duration.Values=[0.0, new_infectivity, new_infectivity, 0.0]
    else:
        raise ValueError( f"profile {profile} not supported at this time." )
    return intervention

def new_intervention_event( camp, new_infectivity, waning="CONSTANT", start_day=1, node_ids=None ):
    """
    Create new NodeInfectivityModifying intervention as scheduled campaign event.
    """
    new_iv = new_intervention( camp, new_infectivity, profile=waning )

    coordinator = s2c.get_class_with_defaults( "StandardEventCoordinator", camp.schema_path )
    coordinator.Intervention_Config = new_iv

    event = s2c.get_class_with_defaults( "CampaignEvent", camp.schema_path )
    event.Event_Coordinator_Config = coordinator

    return event

def new_intervention_as_file( camp, timestep, filename=None ):
    """
    Create new NodeInfectivityModifying intervention as sole scheduled campaign event inside working campaign json file.
    """
    camp.add( new_intervention_event( camp, 0.44, waning="CONSTANT", start_day=timestep, node_ids=[0] ), first=True )
    if filename is None:
        filename = "node_infectivity.json"
    camp.save( filename )
    return filename
